



#include <linux/kernel.h>
#include "mt2060_basic.h"
#include "MT2060Struct.h"
#include "dvb_frontend.h"
#include "rtd2830_priv.h"




/*  Version of this module                         */
#define VERSION 10105             /*  Version 01.15 */

//#define __FLOATING_POINT__

/*
**  The expected version of MT_AvoidSpursData_t
**  If the version is different, an updated file is needed from Microtune
*/
/* Expecting version 1.20 of the Spur Avoidance API */
#define EXPECTED_MT_AVOID_SPURS_INFO_VERSION  010200

#if MT_AVOID_SPURS_INFO_VERSION < EXPECTED_MT_AVOID_SPURS_INFO_VERSION
#error Contact Microtune for a newer version of MT_SpurAvoid.c
#elif MT_AVOID_SPURS_INFO_VERSION > EXPECTED_MT_AVOID_SPURS_INFO_VERSION
#error Contact Microtune for a newer version of mt2060.c
#endif

#ifndef MT2060_CNT
#error You must define MT2060_CNT in the "mt_userdef.h" file
#endif

/*
**  RF Band cross-over frequencies
*/
static UData_t MT2060_RF_Bands[] = 
{
	95000000L,      /*    0 ..  95 MHz: b1011      */
	180000000L,     /*   95 .. 180 MHz: b1010      */
	260000000L,     /*  180 .. 260 MHz: b1001      */
	335000000L,     /*  260 .. 335 MHz: b1000      */
	425000000L,     /*  335 .. 425 MHz: b0111      */
	490000000L,     /*  425 .. 490 MHz: b0110      */
	570000000L,     /*  490 .. 570 MHz: b0101      */
	645000000L,     /*  570 .. 645 MHz: b0100      */
	730000000L,     /*  645 .. 730 MHz: b0011      */
	810000000L      /*  730 .. 810 MHz: b0010      */
	/*  810 ..     MHz: b0001      */
};

/*
** Constants used by the tuning algorithm
*/
#define REF_FREQ          (16000000UL)  /* Reference oscillator Frequency (in Hz) */
#define IF1_FREQ        (1220000000UL)  /* Default IF1 Frequency (in Hz) */
#define IF1_BW            (22000000UL)  /* The IF1 filter bandwidth (in Hz) */
#define TUNE_STEP_SIZE       (50000UL)  /* Tune in steps of 50 kHz */
#define SPUR_STEP_HZ        (250000UL)  /* Step size (in Hz) to move IF1 when avoiding spurs */
#define LO1_STEP_SIZE       (250000UL)  /* Step size (in Hz) of PLL1 */
#define ZIF_BW             (3000000UL)  /* Zero-IF spur-free bandwidth (in Hz) */
#define MAX_HARMONICS_1          (5UL)  /* Highest intra-tuner LO Spur Harmonic to be avoided */ 
#define MAX_HARMONICS_2          (5UL)  /* Highest inter-tuner LO Spur Harmonic to be avoided */ 
#define MIN_LO_SEP         (1000000UL)  /* Minimum inter-tuner LO frequency separation */
#define LO1_FRACN_AVOID    (2999999UL)  /* LO1 FracN numerator avoid region (in Hz) */
#define LO2_FRACN_AVOID      (99999UL)  /* LO2 FracN numerator avoid region (in Hz) */
#define MIN_FIN_FREQ      (48000000UL)  /* Minimum input frequency (in Hz) */
#define MAX_FIN_FREQ     (860000000UL)  /* Maximum input frequency (in Hz) */
#define MIN_FOUT_FREQ     (30000000UL)  /* Minimum output frequency (in Hz) */
#define MAX_FOUT_FREQ     (60000000UL)  /* Maximum output frequency (in Hz) */
#define MIN_DNC_FREQ    (1041000000UL)  /* Minimum LO2 frequency (in Hz) */
#define MAX_DNC_FREQ    (1310000000UL)  /* Maximum LO2 frequency (in Hz) */
#define MIN_UPC_FREQ    (1088000000UL)  /* Minimum LO1 frequency (in Hz) */
#define MAX_UPC_FREQ    (2214000000UL)  /* Maximum LO1 frequency (in Hz) */

UData_t MT2060_Open(
					UData_t MT2060_Addr,
                    Handle_t* hMT2060,
                    Handle_t hUserData
					)
{
   
	UData_t status = MT_OK;             
	MT2060_Info_t* pInfo = NULL;
	struct dvb_frontend* fe = (struct dvb_frontend*)hUserData;
	struct rtd2830_state* p_state = (struct rtd2830_state*)fe->demodulator_priv;
	TUNER_MT2060* mt2060_tuner = (TUNER_MT2060 *)(p_state->pTuner);

	printk("MT2060_Open -- Start\n");
	
	if (hMT2060 == NULL)
		return MT_ARG_NULL;
	
	pInfo = &(mt2060_tuner->MT2060_Info[0]);
	
	status |= MT_RegisterTuner(&pInfo->AS_Data);
	
 	if(MT_NO_ERROR(status))
	{
		printk("MT2060_Open:MT_NO_ERROR -->MT2060_ReInit\n");
        
		pInfo->handle = (Handle_t)pInfo;
		pInfo->hUserData = hUserData;
		pInfo->address = MT2060_Addr;
		status |= MT2060_ReInit((Handle_t) pInfo);
	}
	
	if(MT_IS_ERROR(status))
	/*  MT2060_Close handles the un-registration of the tuner  */
	{ 
		printk("MT2060_Open:MT2060_Close handles the un-registration of the tuner\n");
		MT2060_Close((Handle_t) pInfo);
	}
	else
	{   
		*hMT2060 = pInfo->handle;
	}
	
	printk("MT2060_Open -- end\n");
	return (status);
}


static UData_t IsValidHandle(MT2060_Info_t* handle)
{
	return ((handle != NULL) && (handle->handle == handle)) ? 1 : 0;
}

//  Description:    Release the handle to the tuner.
UData_t MT2060_Close(Handle_t hMT2060)
{
 	MT2060_Info_t* pInfo = (MT2060_Info_t*) hMT2060;
	
	if (!IsValidHandle(pInfo))
		return MT_INV_HANDLE;
	
	/* Unregister tuner with SpurAvoidance routines (if needed) */
	MT_UnRegisterTuner(&pInfo->AS_Data);

	pInfo->handle = NULL;
	pInfo->address = MAX_UDATA;
	pInfo->hUserData = NULL;
	
	return MT_OK;
}

UData_t MT2060_ReInit(Handle_t h)
{ 
	U8Data defaults[] =
    {
            0x3F,            /* reg[0x01] <- 0x3F */
			0x74,            /* reg[0x02] <- 0x74 */
			0x80,            /* reg[0x03] <- 0x80 (FM1CA = 1) */
			0x08,            /* reg[0x04] <- 0x08 */
			0x93,            /* reg[0x05] <- 0x93 */
			0x88,            /* reg[0x06] <- 0x88 (RO) */
			0x80,            /* reg[0x07] <- 0x80 (RO) */
			0x60,            /* reg[0x08] <- 0x60 (RO) */
			0x20,            /* reg[0x09] <- 0x20 */
			0x1E,            /* reg[0x0A] <- 0x1E */
			0x31,            /* reg[0x0B] <- 0x31 (VGAG = 1) */
			0xFF,            /* reg[0x0C] <- 0xFF */
			0x80,            /* reg[0x0D] <- 0x80 */
			0xFF,            /* reg[0x0E] <- 0xFF */
			0x00,            /* reg[0x0F] <- 0x00 */
			0x2C,            /* reg[0x10] <- 0x2C (HW def = 3C) */
			0x42,            /* reg[0x11] <- 0x42 */
    };
	
	
	MT2060_Info_t* pInfo = (MT2060_Info_t*) h;
	struct dvb_frontend* fe = ( struct dvb_frontend* )(pInfo->hUserData);
	struct rtd2830_state* p_state= (struct rtd2830_state*)fe->demodulator_priv;
	TUNER_MT2060* mt2060_tuner = (TUNER_MT2060 *)(p_state->pTuner);


	UData_t              status = MT_OK;                  /* Status to be returned        */
	SData_t              i = 0;
	U8Data               read_time = 0;
	
	printk("MT2060_ReInit -- start\n");
    /*  Verify that the handle passed points to a valid tuner         */
    if(IsValidHandle(pInfo) == 0)
	{
		status |= MT_INV_HANDLE;
		return status;
	}
	
    //  Read the Part/Rev code from the tuner, жtunerǷȷ
    if(MT_NO_ERROR(status))
    {
		status |= MT_ReadSub(pInfo->hUserData,
			pInfo->address,
			PART_REV, 
			pInfo->reg, 
			1
			);
	} 	
    if(!MT_NO_ERROR(status))
	{
		return status;
	}
	if(pInfo->reg[0] != 0x63) 
	{
		printk(" ERROR TUNER ID -- id = 0x%x\n", pInfo->reg[0]);
		status |= MT_TUNER_ID_ERR;      /*  Wrong tuner Part/Rev code   */
	}
	
    if(MT_NO_ERROR(status))
    {
        /*  Initialize the tuner state.  */
		printk("MT2060_ReInit:Initialize the tuner state.\n");
      
		pInfo->version = VERSION;
        pInfo->tuner_id = pInfo->reg[0];
        pInfo->AS_Data.f_ref = REF_FREQ;
        pInfo->AS_Data.f_in = 533000000UL;
        pInfo->AS_Data.f_if1_Center = mt2060_tuner->MT2060_FMF;// IF1_CENTER;
        pInfo->AS_Data.f_if1_bw = IF1_BW;
        pInfo->AS_Data.f_out = 36125000UL;                         //43750000UL;
        pInfo->AS_Data.f_out_bw = 8750000UL;                       //6750000UL;
        pInfo->AS_Data.f_zif_bw = ZIF_BW;
        pInfo->AS_Data.f_LO1_Step = LO1_STEP_SIZE;
        pInfo->AS_Data.f_LO2_Step = TUNE_STEP_SIZE;
        pInfo->AS_Data.maxH1 = MAX_HARMONICS_1;
        pInfo->AS_Data.maxH2 = MAX_HARMONICS_2;
        pInfo->AS_Data.f_min_LO_Separation = MIN_LO_SEP;
        pInfo->AS_Data.f_if1_Request = mt2060_tuner->MT2060_FMF;//IF1_CENTER;
        pInfo->AS_Data.f_LO1 = 1871000000UL;
        pInfo->AS_Data.f_LO2 = 1176250000UL;
        pInfo->f_IF1_actual = pInfo->AS_Data.f_LO1 - pInfo->AS_Data.f_in;
        pInfo->AS_Data.f_LO1_FracN_Avoid = LO1_FRACN_AVOID;
        pInfo->AS_Data.f_LO2_FracN_Avoid = LO2_FRACN_AVOID;
		
        pInfo->num_regs = END_REGS;
        for ( i = 0; i < 10; i++ )
        {
            pInfo->RF_Bands[i] = MT2060_RF_Bands[i];
        }
		
        /*  Write the default values to each of the tuner registers.  */
		printk("MT2060_ReInit:Write the default values to each of the tuner registers\n");
        status |= MT_WriteSub(pInfo->hUserData,
			pInfo->address,
			LO1C_1,
			defaults, 
			sizeof(defaults)
			);
    }
	
    //  Read back all the registers from the tuner 
    if(MT_NO_ERROR(status))
	{
		status |= MT_ReadSub(pInfo->hUserData, 
		pInfo->address,
		PART_REV,
		pInfo->reg,
		END_REGS
		);
	}	
	if(!MT_NO_ERROR(status))
	{
		printk("failed to ReInit MT2060 tuner\n");
		return status;
	}
//	for(i=0; i<END_REGS; i++)
//	{
//		printk(("reg[%d] = 0x%x\n", i, pInfo->reg[i]));
//	}

    //  Wait for the MT2060 to finish initializing  
    while((MT_NO_ERROR(status)) && ((pInfo->reg[MISC_STATUS] & 0x40) == 0x00))
	{
		MT_Sleep(pInfo->hUserData, 20);

		status |= MT_ReadSub(pInfo->hUserData,
			pInfo->address,
			MISC_STATUS,
			&pInfo->reg[MISC_STATUS], 
			1
			);
		
		if(read_time++ > 100)
		{
			/*  Timed out waiting for MT2060 to finish initializing  */
			printk("***MT2060_ReInit:: Timed out waiting for MT2060 to finish initializing*********\n");
			status |= MT_TUNER_INIT_ERR;
			break;
		}
//		printk(("MISC_STATUS DATA = 0x%x, status = %d", pInfo->reg[MISC_STATUS], status));		
	}
	
	printk("******************** read time = %d ******************\n", read_time);
    return (status);
}

UData_t MT2060_SetGainRange(Handle_t h, UData_t range)
{

    UData_t         status = MT_OK;                  /* Status to be returned        */
    MT2060_Info_t * pInfo = (MT2060_Info_t *) h;

    printk("MT2060_SetGainRange\n");
	
    if(IsValidHandle(pInfo) == 0)
        return MT_INV_HANDLE;
	
    switch (range)
    {
    case MT2060_VGAG_0DB:
    case MT2060_VGAG_5DB:
    case MT2060_VGAG_11DB:
        pInfo->reg[MISC_CTRL_3] = ((pInfo->reg[MISC_CTRL_3] & 0xFC) | (U8Data) range);
        status = MT_WriteSub(pInfo->hUserData,
			pInfo->address,
			MISC_CTRL_3,
			&pInfo->reg[MISC_CTRL_3],
			1
			);
        break;
    
	default:
        status = MT_ARG_RANGE;
    }
	
    return (status);
}

UData_t MT2060_SetParam(Handle_t     h,
                        MT2060_Param param,
                        UData_t      nValue)
{
    UData_t status = MT_OK;                  /* Status to be returned        */
    MT2060_Info_t* pInfo = (MT2060_Info_t*) h;
    
    printk("MT2060_SetParam\n");
	
    /*  Verify that the handle passed points to a valid tuner         */
    if (IsValidHandle(pInfo) == 0)
        status |= MT_INV_HANDLE;
	
    if (MT_NO_ERROR(status))
    {
        switch (param)
        {
			/*  crystal frequency                     */
        case MT2060_SRO_FREQ:
            pInfo->AS_Data.f_ref = nValue;
            pInfo->AS_Data.f_LO1_FracN_Avoid = 3 * nValue / 16 - 1;
            pInfo->AS_Data.f_LO2_FracN_Avoid = nValue / 160 - 1;
            pInfo->AS_Data.f_LO1_Step = nValue / 64;
            break;
			
			/*  minimum tuning step size              */
        case MT2060_STEPSIZE:
            pInfo->AS_Data.f_LO2_Step = nValue;
            break;
			
			/*  Center of input channel               */
        case MT2060_INPUT_FREQ:
            pInfo->AS_Data.f_in = nValue;
            break;
			
			/*  LO1 minimum step size                 */
        case MT2060_LO1_STEPSIZE:
            pInfo->AS_Data.f_LO1_Step = nValue;
            break;
			
			/*  LO1 FracN keep-out region             */
        case MT2060_LO1_FRACN_AVOID:
            pInfo->AS_Data.f_LO1_FracN_Avoid = nValue;
            break;
			
			/*  Requested 1st IF                      */
        case MT2060_IF1_REQUEST:
            pInfo->AS_Data.f_if1_Request = nValue;
            break;
			
			/*  Center of 1st IF SAW filter           */
        case MT2060_IF1_CENTER:
            pInfo->AS_Data.f_if1_Center = nValue;
            break;
			
			/*  Bandwidth of 1st IF SAW filter        */
        case MT2060_IF1_BW:
            pInfo->AS_Data.f_if1_bw = nValue;
            break;
			
			/*  zero-IF bandwidth                     */
        case MT2060_ZIF_BW:
            pInfo->AS_Data.f_zif_bw = nValue;
            break;
			
			/*  LO2 minimum step size                 */
        case MT2060_LO2_STEPSIZE:
            pInfo->AS_Data.f_LO2_Step = nValue;
            break;
			
			/*  LO2 FracN keep-out region             */
        case MT2060_LO2_FRACN_AVOID:
            pInfo->AS_Data.f_LO2_FracN_Avoid = nValue;
            break;
			
			/*  output center frequency               */
        case MT2060_OUTPUT_FREQ:
            pInfo->AS_Data.f_out = nValue;
            break;
			
			/*  output bandwidth                      */
        case MT2060_OUTPUT_BW:
            pInfo->AS_Data.f_out_bw = nValue;
            break;
			
			/*  min inter-tuner LO separation         */
        case MT2060_LO_SEPARATION:
            pInfo->AS_Data.f_min_LO_Separation = nValue;
            break;
			
			/*  max # of intra-tuner harmonics        */
        case MT2060_MAX_HARM1:
            pInfo->AS_Data.maxH1 = nValue;
            break;
			
			/*  max # of inter-tuner harmonics        */
        case MT2060_MAX_HARM2:
            pInfo->AS_Data.maxH2 = nValue;
            break;
			
			/*  These parameters are read-only  */
        case MT2060_IC_ADDR:
        case MT2060_MAX_OPEN:
        case MT2060_NUM_OPEN:
        case MT2060_LO1_FREQ:
        case MT2060_IF1_ACTUAL:
        case MT2060_LO2_FREQ:
        case MT2060_AS_ALG:
        case MT2060_EXCL_ZONES:
        case MT2060_SPUR_AVOIDED:
        case MT2060_NUM_SPURS:
        case MT2060_SPUR_PRESENT:
        case MT2060_EOP:
        default:
            status |= MT_ARG_RANGE;
        }
    }
    return (status);
}

static UData_t Round_fLO(UData_t f_LO, UData_t f_LO_Step, UData_t f_ref)
{
    return f_ref * (f_LO / f_ref)
        + f_LO_Step * (((f_LO % f_ref) + (f_LO_Step / 2)) / f_LO_Step);
}

static UData_t CalcLO1Mult(UData_t *Div,
                           UData_t *FracN,
                           UData_t  f_LO,
                           UData_t  f_LO_Step,
                           UData_t  f_Ref)
{
    /*  Calculate the whole number portion of the divider */
    *Div = f_LO / f_Ref;
	
    /*  Calculate the numerator value (round to nearest f_LO_Step) */
    *FracN = (64 * (((f_LO % f_Ref) + (f_LO_Step / 2)) / f_LO_Step)+ (f_Ref / f_LO_Step / 2)) / (f_Ref / f_LO_Step);
	
    return (f_Ref * (*Div)) + (f_LO_Step * (*FracN));
}

static UData_t CalcLO2Mult(UData_t *Div,
                           UData_t *FracN,
                           UData_t  f_LO,
                           UData_t  f_LO_Step,
                           UData_t  f_Ref)
{
    const UData_t FracN_Scale = (f_Ref / (MAX_UDATA >> 13)) + 1;
	
    /*  Calculate the whole number portion of the divider */
    *Div = f_LO / f_Ref;
	
    /*  Calculate the numerator value (round to nearest f_LO_Step) */
    *FracN = (8191 * (((f_LO % f_Ref) + (f_LO_Step / 2)) / f_LO_Step) + (f_Ref / f_LO_Step / 2)) / (f_Ref / f_LO_Step);
	
    return (f_Ref * (*Div))
		+ FracN_Scale * (((f_Ref / FracN_Scale) * (*FracN)) / 8191);
	
}

static UData_t LO1LockWorkAround(MT2060_Info_t *pInfo)
{
    U8Data cs;
    U8Data tad;
    UData_t status = MT_OK;                  /* Status to be returned        */
    SData_t ChkCnt = 0;
	
    cs = 30;  /* Pick starting CapSel Value */
	
    pInfo->reg[LO1_1] &= 0x7F;  /* Auto CapSelect off */
    pInfo->reg[LO1_2] = (pInfo->reg[LO1_2] & 0xC0) | cs;
    status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO1_1, &pInfo->reg[LO1_1], 2);  /* 0x0C - 0x0D */
	
    while ((ChkCnt < 64) && (MT_NO_ERROR(status)))
    {
        MT_Sleep(pInfo->hUserData, 2);
        status |= MT_ReadSub(pInfo->hUserData, pInfo->address, LO_STATUS, &pInfo->reg[LO_STATUS], 1);
        tad = (pInfo->reg[LO_STATUS] & 0x70) >> 4;  /* LO1AD */
        if (MT_NO_ERROR(status))
        {
            if (tad == 0)   /* Found good spot -- quit looking */
                break;
            else if (tad & 0x04 )  /* either 4 or 5; decrement */
            {
                pInfo->reg[LO1_2] = (pInfo->reg[LO1_2] & 0xC0) | --cs;
                status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO1_2, &pInfo->reg[LO1_2], 1);   /* 0x0D */
            }
            else if (tad & 0x02 )  /* either 2 or 3; increment */
            {
                pInfo->reg[LO1_2] = (pInfo->reg[LO1_2] & 0xC0) | ++cs;
                status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO1_2, &pInfo->reg[LO1_2], 1);   /* 0x0D */
            }
            ChkCnt ++;  /* Count this attempt */
        }
    }
    if (MT_NO_ERROR(status))
    {
        status |= MT_ReadSub(pInfo->hUserData, pInfo->address, LO_STATUS, &pInfo->reg[LO_STATUS], 1);
    }
	
    if (MT_NO_ERROR(status))
    {
        if ((pInfo->reg[LO_STATUS] & 0x80) == 0x00)
            status |= MT_UPC_UNLOCK;
		
        if ((pInfo->reg[LO_STATUS] & 0x08) == 0x00)
            status |= MT_DNC_UNLOCK;
    }
	
    return (status);
}

UData_t MT2060_GetLocked(Handle_t h)
{
    const UData_t nMaxWait = 200;            /*  wait a maximum of 200 msec   */
    const UData_t nPollRate = 2;             /*  poll status bits every 2 ms */
    const UData_t nMaxLoops = nMaxWait / nPollRate;
    UData_t status = MT_OK;                  /* Status to be returned        */
    UData_t nDelays = 0;
    MT2060_Info_t* pInfo = (MT2060_Info_t*) h;
	
    if (IsValidHandle(pInfo) == 0)
        return MT_INV_HANDLE;
	
    do
    {
        status |= MT_ReadSub(pInfo->hUserData, pInfo->address, LO_STATUS, &pInfo->reg[LO_STATUS], 1);
		
        if ((MT_IS_ERROR(status)) || ((pInfo->reg[LO_STATUS] & 0x88) == 0x88))
            return (status);
		
        MT_Sleep(pInfo->hUserData, nPollRate);       /*  Wait between retries  */
    }
    while (++nDelays < nMaxLoops);
	
    if ((pInfo->reg[LO_STATUS] & 0x80) == 0x00)
        status |= MT_UPC_UNLOCK;
    if ((pInfo->reg[LO_STATUS] & 0x08) == 0x00)
        status |= MT_DNC_UNLOCK;
	
    return (status);
}

UData_t MT2060_ChangeFreq(Handle_t h,
                          UData_t f_in,     /* RF input center frequency   */
                          UData_t f_IF1,    /* desired 1st IF frequency    */
                          UData_t f_out,    /* IF output center frequency  */
                          UData_t f_IFBW)   /* IF output bandwidth + guard */
{
    
    MT2060_Info_t* pInfo = (MT2060_Info_t*) h;
    UData_t status = MT_OK;       /*  status of operation             */
    UData_t t_status;             /*  temporary status of operation   */
    UData_t LO1;                  /*  1st LO register value           */
    UData_t Num1;                 /*  Numerator for LO1 reg. value    */
    UData_t LO2;                  /*  2nd LO register value           */
    UData_t Num2;                 /*  Numerator for LO2 reg. value    */
    UData_t ofLO1, ofLO2;         /*  last time's LO frequencies      */
    UData_t center;
    UData_t RFBand;               /*  RF Band setting                 */
    UData_t idx;                  /*  index loop                      */

    printk("MT2060_ChangeFreq -- start\n");	
    printk("f_in = %d, f_IF1 = %d, f_out = %d, f_IFBW = %d", f_in, f_IF1, f_out, f_IFBW);

	/*  Verify that the handle passed points to a valid tuner         */
    if(IsValidHandle(pInfo) == 0)
	{
		printk("Invalid Handle\n");
		return MT_INV_HANDLE;
	}
	
    /*  Check the input and output frequency ranges                   */
    if ((f_in < MIN_FIN_FREQ) || (f_in > MAX_FIN_FREQ))
	{
		printk("invalid frequency\n");
		status |= MT_FIN_RANGE;
		return status;
	}

    if ((f_out < MIN_FOUT_FREQ) || (f_out > MAX_FOUT_FREQ))
    {
		printk("invalid out frequency\n");		
		status |= MT_FOUT_RANGE;
		return status;
	}
	
		/*
		**  Save original LO1 and LO2 register values
    */
    ofLO1 = pInfo->AS_Data.f_LO1;
    ofLO2 = pInfo->AS_Data.f_LO2;	
    /*
    **  Find RF Band setting
    */
    RFBand = 1;                        /*  def when f_in > all    */
    for (idx=0; idx<10; ++idx)
    {
        if (pInfo->RF_Bands[idx] >= f_in)
        {
            RFBand = 11 - idx;
            break;
        }
    }
	//    VERBOSE_PRINT1("Using RF Band #%d.\n", RFBand);
    /*
    **  Assign in the requested values
    */
    pInfo->AS_Data.f_in = f_in;
    pInfo->AS_Data.f_out = f_out;
    pInfo->AS_Data.f_out_bw = f_IFBW;
    //  Request a 1st IF such that LO1 is on a step size
	printk("Round_fLO\n");
	pInfo->AS_Data.f_if1_Request = Round_fLO(f_IF1 + f_in, pInfo->AS_Data.f_LO1_Step, pInfo->AS_Data.f_ref) - f_in;
	
    /*
    **  Calculate frequency settings.  f_IF1_FREQ + f_in is the
    **  desired LO1 frequency
    */
	printk("MT_ResetExclZones\n");
    MT_ResetExclZones(&pInfo->AS_Data);
	
    if (((f_in % pInfo->AS_Data.f_ref) <= f_IFBW/2)
        || ((pInfo->AS_Data.f_ref - (f_in % pInfo->AS_Data.f_ref)) <= f_IFBW/2))
    {
	/*
	**  Exclude LO frequencies that allow SRO harmonics to pass through
        */
        center = pInfo->AS_Data.f_ref * ((pInfo->AS_Data.f_if1_Center - pInfo->AS_Data.f_if1_bw/2 + pInfo->AS_Data.f_in) / pInfo->AS_Data.f_ref) - pInfo->AS_Data.f_in;
        while (center < pInfo->AS_Data.f_if1_Center + pInfo->AS_Data.f_if1_bw/2 + pInfo->AS_Data.f_LO1_FracN_Avoid)
        {
            //  Exclude LO1 FracN
            MT_AddExclZone(&pInfo->AS_Data, center-pInfo->AS_Data.f_LO1_FracN_Avoid, center+pInfo->AS_Data.f_LO1_FracN_Avoid);
            center += pInfo->AS_Data.f_ref;
        }
		
        center = pInfo->AS_Data.f_ref * ((pInfo->AS_Data.f_if1_Center - pInfo->AS_Data.f_if1_bw/2 - pInfo->AS_Data.f_out) / pInfo->AS_Data.f_ref) + pInfo->AS_Data.f_out;
        while (center < pInfo->AS_Data.f_if1_Center + pInfo->AS_Data.f_if1_bw/2 + pInfo->AS_Data.f_LO2_FracN_Avoid)
        {
            //  Exclude LO2 FracN
            MT_AddExclZone(&pInfo->AS_Data, center-pInfo->AS_Data.f_LO2_FracN_Avoid, center+pInfo->AS_Data.f_LO2_FracN_Avoid);
            center += pInfo->AS_Data.f_ref;
        }
    }
	
    f_IF1 = MT_ChooseFirstIF(&pInfo->AS_Data);
    pInfo->AS_Data.f_LO1 = Round_fLO(f_IF1 + f_in, pInfo->AS_Data.f_LO1_Step, pInfo->AS_Data.f_ref);
    pInfo->AS_Data.f_LO2 = Round_fLO(pInfo->AS_Data.f_LO1 - f_out - f_in, pInfo->AS_Data.f_LO2_Step, pInfo->AS_Data.f_ref);
	
    /*
    ** Check for any LO spurs in the output bandwidth and adjust
    ** the LO settings to avoid them if needed
    */
    status |= MT_AvoidSpurs(h, &pInfo->AS_Data);
    printk("MT_AvoidSpurs status = %d", status);
	
    /*
    ** MT_AvoidSpurs spurs may have changed the LO1 & LO2 values.
    ** Recalculate the LO frequencies and the values to be placed
    ** in the tuning registers.
    */
    pInfo->AS_Data.f_LO1 = CalcLO1Mult(&LO1, &Num1, pInfo->AS_Data.f_LO1, pInfo->AS_Data.f_LO1_Step, pInfo->AS_Data.f_ref);
    pInfo->AS_Data.f_LO2 = Round_fLO(pInfo->AS_Data.f_LO1 - f_out - f_in, pInfo->AS_Data.f_LO2_Step, pInfo->AS_Data.f_ref);
    pInfo->AS_Data.f_LO2 = CalcLO2Mult(&LO2, &Num2, pInfo->AS_Data.f_LO2, pInfo->AS_Data.f_LO2_Step, pInfo->AS_Data.f_ref);
	
    /*
    **  If we have the same LO frequencies and we're already locked,
    **  then just return without writing any registers.
    */
    printk("ofLO1=%d, ofLO2=%d, pInfo->reg[LO_STATUS]=%d \n",ofLO1, ofLO2, pInfo->reg[LO_STATUS]);
    printk("status1 = %d\n", status);
//    if ((ofLO1 == pInfo->AS_Data.f_LO1) 
//        && (ofLO2 == pInfo->AS_Data.f_LO2) 
//        && ((pInfo->reg[LO_STATUS] & 0x88) == 0x88))
//    {
//        pInfo->f_IF1_actual = pInfo->AS_Data.f_LO1 - f_in;
//        return (status);
//    }
	
    /*
    **  Check the upconverter and downconverter frequency ranges
    */
    if ((pInfo->AS_Data.f_LO1 < MIN_UPC_FREQ) || (pInfo->AS_Data.f_LO1 > MAX_UPC_FREQ))
        status |= MT_UPC_RANGE;
    printk("status2 = %d\n", status);
	
    if ((pInfo->AS_Data.f_LO2 < MIN_DNC_FREQ) || (pInfo->AS_Data.f_LO2 > MAX_DNC_FREQ))
        status |= MT_DNC_RANGE;
    printk("status3 = %d\n", status);
	
		/*
		**  Make sure that the Auto CapSelect is turned on since it
		**  may have been turned off last time we tuned.
    */
    printk("status = %d, reg[LO1_1] = 0x%x", status, pInfo->reg[LO1_1]);
    if(MT_NO_ERROR(status) && ((pInfo->reg[LO1_1] & 0x80) == 0x00))
    {
        pInfo->reg[LO1_1] |= 0x80;

        printk("Write LO1_1 register\n");
        status |= MT_WriteSub(pInfo->hUserData, 
			pInfo->address,
			LO1_1, 
			&pInfo->reg[LO1_1],
			1
			);   /* 0x0C */
    }
	
    /*
    **  Place all of the calculated values into the local tuner
    **  register fields.
    */
	printk("status4 = %d\n", status);
	if(MT_NO_ERROR(status))
    {
        pInfo->reg[LO1C_1] = (U8Data)((RFBand << 4) | (Num1 >> 2));
        pInfo->reg[LO1C_2] = (U8Data)(LO1 & 0xFF);
		
        pInfo->reg[LO2C_1] = (U8Data)((Num2 & 0x000F) | ((Num1 & 0x03) << 4));
        pInfo->reg[LO2C_2] = (U8Data)((Num2 & 0x0FF0) >> 4);
        pInfo->reg[LO2C_3] = (U8Data)(((LO2 << 1) & 0xFE) | ((Num2 & 0x1000) >> 12));
		
        /*
        ** Now write out the computed register values
        ** IMPORTANT: There is a required order for writing some of the
        **            registers.  R2 must follow R1 and  R5 must
        **            follow R3 & R4. Simple numerical order works, so...
        */

		printk("Write LO1C_1 register\n");
		
        status |= MT_WriteSub(pInfo->hUserData, 
			pInfo->address,
			LO1C_1, 
			&pInfo->reg[LO1C_1], 
			5
			);   /* 0x01 - 0x05 */
    }
	
    /*
    **  Check for LO's locking
    */
    if (MT_NO_ERROR(status))
    {
        t_status = MT2060_GetLocked(h);
        if ((t_status & (MT_UPC_UNLOCK | MT_DNC_UNLOCK)) == MT_UPC_UNLOCK)
            /*  Special Case: LO1 not locked, LO2 locked  */
            status |= (t_status & ~MT_UPC_UNLOCK) | LO1LockWorkAround(pInfo);
        else
            /*  OR in the status from the call to MT2060_GetLocked()  */
            status |= t_status;
    }
	
    /*
    **  If we locked OK, assign calculated data to MT2060_Info_t structure
    */
    if (MT_NO_ERROR(status))
    {
        pInfo->f_IF1_actual = pInfo->AS_Data.f_LO1 - f_in;
    }
	
    printk("MT2060_ChangeFreq -- end\n");
    return (status);
}

UData_t MT2060_GetFMF(Handle_t h)
{
    UData_t status = MT_OK;                  /* Status to be returned        */
    UData_t idx;                             /* loop index variable          */
    MT2060_Info_t* pInfo = (MT2060_Info_t*) h;
	
    if (IsValidHandle(pInfo) == 0)
        return MT_INV_HANDLE;
	
    /*  Set the FM1 Single-Step bit  */
    pInfo->reg[LO2C_1] |= 0x40;
    status = MT_WriteSub(pInfo->hUserData, pInfo->address, LO2C_1, &pInfo->reg[LO2C_1], 1);
	
    if (MT_NO_ERROR(status))
    {
        /*  Set the FM1CA bit to start the FMF location  */
        pInfo->reg[LO2C_1] |= 0x80;
        status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO2C_1, &pInfo->reg[LO2C_1], 1);
        pInfo->reg[LO2C_1] &= 0x7F;    /*  FM1CA bit automatically goes back to 0  */
		
        /*  Toggle the single-step 8 times  */
        for (idx=0; (idx<8) && (MT_NO_ERROR(status)); ++idx)
        {
            MT_Sleep(pInfo->hUserData, 20);
            /*  Clear the FM1 Single-Step bit  */
            pInfo->reg[LO2C_1] &= 0xBF;
            status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO2C_1, &pInfo->reg[LO2C_1], 1);
			
            MT_Sleep(pInfo->hUserData, 20);
            /*  Set the FM1 Single-Step bit  */
            pInfo->reg[LO2C_1] |= 0x40;
            status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO2C_1, &pInfo->reg[LO2C_1], 1);
        }
    }
	
    /*  Clear the FM1 Single-Step bit  */
    pInfo->reg[LO2C_1] &= 0xBF;
    status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO2C_1, &pInfo->reg[LO2C_1], 1);
	
    if (MT_NO_ERROR(status))
    {
        idx = 25;
        status |= MT_ReadSub(pInfo->hUserData, pInfo->address, MISC_STATUS, &pInfo->reg[MISC_STATUS], 1);
		
        /*  Poll FMCAL until it is set or we time out  */
        while ((MT_NO_ERROR(status)) && (--idx) && (pInfo->reg[MISC_STATUS] & 0x40) == 0)
        {
            MT_Sleep(pInfo->hUserData, 20);
            status = MT_ReadSub(pInfo->hUserData, pInfo->address, MISC_STATUS, &pInfo->reg[MISC_STATUS], 1);
        }
		
        if ((MT_NO_ERROR(status)) && ((pInfo->reg[MISC_STATUS] & 0x40) != 0))
        {
            /*  Read the 1st IF center offset register  */
            status = MT_ReadSub(pInfo->hUserData, pInfo->address, FM_FREQ, &pInfo->reg[FM_FREQ], 1);
            if (MT_NO_ERROR(status))
            {
                pInfo->AS_Data.f_if1_Center = 1000000L * (1084 + pInfo->reg[FM_FREQ]);
            }
        }
    }
	
    return (status);
}

#if defined (__FLOATING_POINT__)
#include <math.h>

typedef double          FData_t;        /*  floating point data type        */

static void spline(FData_t x[], FData_t y[], FData_t y2[])
{
    int i;
    FData_t dx[8],dy[8],w[8],d[8];
    
    dx[0] = x[1]-x[0];
    dy[0] = y[1]-y[0];
    d[0] = dx[0] * dx[0];       //  Assumes 2(x[2]-x[0]) is much greater than 1
    y2[0] = y2[8] = 0.0;
    w[0] = 0.0;
	
    /*  Notes for further optimization:
    **
    **  d[0] = 6889000000000000.0
    **  d[1] = 195999999.00000000
    **  d[2] = 58852040.810469598
    **  d[3] = 56176853.055536300
    **  d[4] = 55994791.666639537
    **  d[5] = 55981769.137752682
    **  d[6] = 55980834.413318574
    **  d[7] = 191980767.30441749
    **  dx[0]=dx[7] = 83000000
    **  dx[1..6] = 15000000  (15 MHz spacing from x[1] through x[7])
    **
    */
    for (i=1;i<8;i++)
    {
        dx[i] = (x[i+1] - x[i]);
        dy[i] = (y[i+1] - y[i]);
        w[i] = 6*((dy[i] / dx[i]) - (dy[i-1] / dx[i-1])) - (w[i-1] * dx[i-1] / d[i-1]);
        d[i] = 2 * (x[i+1] - x[i-1]) - dx[i-1] * dx[i-1] / d[i-1];
    }
	
    for (i=7;i>0;i--)
        y2[i] = (w[i] - dx[i]*y2[i+1]) / d[i];
}

static FData_t findMax(FData_t x[], FData_t y[], FData_t y2[], FData_t *xmax)
{
    FData_t ymax = y[0];
    UData_t i;
	
    /*  Calculate the maximum of each cubic spline segment  */
    for (i=0; i<8; i++)
    {
	/*
	**  Compute the cubic polynomial coefficients:
	**
	**  y = ax^3 + bx^2 + cx + d
	**
        */
        FData_t dx = x[i+1] - x[i];
        FData_t dy = y[i+1] - y[i];
        FData_t a = (y2[i+1] - y2[i]) / (6 * dx);
        FData_t b = (x[i+1] * y2[i] - x[i] * y2[i+1]) / (2 * dx);
        FData_t c = (dy + a*(x[i]*x[i]*x[i]-x[i+1]*x[i+1]*x[i+1]) + b*(x[i]*x[i]-x[i+1]*x[i+1])) / dx;
        FData_t d = y[i] - a*x[i]*x[i]*x[i] - b*x[i]*x[i] - c*x[i];
        FData_t t = b*b - 3*a*c;
		
        if (ymax < y[i+1])
        {
            *xmax = x[i+1];
            ymax = y[i+1];
        }
		
        /*
        **  Solve the cubic polynomial's 1st derivative for zero
        **  to find the maxima or minima
        **
        **  y' = 3ax^2 + 2bx + c = 0
        **
        **       -b +/- sqrt(b^2 - 3ac)
        **  x = ---------------------------
        **                3a
        */
		
        /*  Skip this segment if x will be imaginary  */
        if (t>=0.0)
        {
            FData_t x_t;
			
            t = sqrt(t);
            x_t = (-b + t) / (3*a);
            /*  Only check answers that are between x[i] & x[i+1]  */
            if ((x[i]<=x_t) && (x_t<=x[i+1]))
            {
                FData_t y_t = a*x_t*x_t*x_t + b*x_t*x_t + c*x_t + d;
                if (ymax < y_t)
                {
                    *xmax = x_t;
                    ymax = y_t;
                }
            }
			
            x_t = (-b - t) / (3*a);
            /*  Only check answers that are between x[i] & x[i+1]  */
            if ((x[i]<=x_t) && (x_t<=x[i+1]))
            {
                FData_t y_t = a*x_t*x_t*x_t + b*x_t*x_t + c*x_t + d;
                if (ymax < y_t)
                {
                    *xmax = x_t;
                    ymax = y_t;
                }
            }
        }
    }
    return (ymax);
}


static FData_t findRoot(FData_t x[], 
                        FData_t y[], 
                        FData_t y2[], 
                        FData_t x_at_ymax,
                        FData_t x_at_ymin)
{
    FData_t x_upper, x_lower;           /*  limits for zero-crossing       */
    UData_t i;
    FData_t dx, dy;
    FData_t a, b, c, d;                 /*  cubic coefficients             */
    FData_t xg;                         /*  x-axis guesses                 */
    FData_t y_err;                      /*  y-axis error of x-axis guess   */
	
    /*  Find the index into x[] that is just below x_ymax  */
    i = 8;
    while (x[i] > x_at_ymax)
        --i;
	
    if (x_at_ymax > x_at_ymin)
    {
	/*  Decrement i until y[i] < 0
	**  We want to end up with this:
	**
	**             X  {x[i+1], y[i+1]}
	**            /
	**  0 ----------------------
	**          /
	**         /
	**        X  {x[i], y[i]}
	**
        */
        while (y[i] > 0)
            --i;
        x_upper = (x_at_ymax < x[i+1]) ? x_at_ymax : x[i+1];
        x_lower = (x[i] < x_at_ymin) ? x_at_ymin : x[i];
    }
    else
    {
	/*  Increment i until y[i] < 0
	**  We want to end up with this:
	**
	**        X  {x[i], y[i]}
	**         \
	**  0 ----------------------
	**           \
	**            \
	**             X  {x[i+1], y[i+1]}
	**
        */
        while (y[++i] > 0);
        --i;
        x_upper = (x_at_ymin < x[i+1]) ? x_at_ymin : x[i+1];
        x_lower = (x[i] < x_at_ymax) ? x_at_ymax : x[i];
    }
	
    /*
    **  Compute the cubic polynomial coefficients:
    **
    **  y = ax^3 + bx^2 + cx + d
    **
    */
    dx = x[i+1] - x[i];
    dy = y[i+1] - y[i];
    a = (y2[i+1] - y2[i]) / (6 * dx);
    b = (x[i+1] * y2[i] - x[i] * y2[i+1]) / (2 * dx);
    c = (dy + a*(x[i]*x[i]*x[i]-x[i+1]*x[i+1]*x[i+1]) + b*(x[i]*x[i]-x[i+1]*x[i+1])) / dx;
    d = y[i] - a*x[i]*x[i]*x[i] - b*x[i]*x[i] - c*x[i];
	
    /*
    **  Make an initial guess at the root (bisection)
    */
    xg = (x_lower + x_upper) / 2.0;
	
    while (1)
    {
        FData_t u = 3*a*xg*xg+2*b*xg+c;
        FData_t v = 6*a*xg+2*b;
        FData_t t;
		
        /*
        **  Compute the y-axis value of this guess (error)
        */
        y_err = a*xg*xg*xg + b*xg*xg + c*xg + d;
		
        /*  Halt search if we're within 0.01 dB  */
        if (fabs(y_err) < 0.01)
            break;
		
        /*  Update upper and/or lower bound  */
        if ((x_at_ymax > x_at_ymin) ^ (y_err < 0))
            x_upper = xg;
        else
            x_lower = xg;
		
        /*  Check to see if we can use the quadratic form of the root solution  */
        t = u*u - 2*y_err*v;
        if (t>=0)
        {
            UData_t bInBound2, bInBound3;
            FData_t xg2, xg3;
			
            /*  Compute new guesses  */
            t = sqrt(t);
            xg2 = xg - (u - t) / v;
            bInBound2 = (xg2 >= x_lower) && (xg2 <= x_upper);
            xg3 = xg - (u + t) / v;
            bInBound3 = (xg3 >= x_lower) && (xg3 <= x_upper);
			
            if (bInBound2 && bInBound3)
                /*  If both xg2 & xg3 are valid, choose the one closest to x_at_ymax  */
                xg = (fabs(xg2 - x_at_ymax) < fabs(xg3 - x_at_ymax)) ? xg2 : xg3;
			
            else if (bInBound2 != 0)
                xg = xg2;               /*  Keep this new guess  */
			
            else if (bInBound3 != 0)
                xg = xg3;               /*  Keep this new guess  */
			
            else
                /*  Quadratic form failed, use bisection  */
                xg = (x_lower + x_upper) / 2.0;
        }
        else
        {
            /*  Quadratic form failed, use bisection  */
            xg = (x_lower + x_upper) / 2.0;
        }
    }
	
    return (xg);
}
#endif

static UData_t MT2060_Program_LOs(MT2060_Info_t* pInfo,
                                  UData_t f_in,
                                  UData_t f_IF1,
                                  UData_t f_out)
{
    UData_t status = MT_OK;
    UData_t LO1, Num1, LO2, Num2;
    UData_t idx;
    long RFBand = 1;                  /*  RF Band setting  */
	
    if (f_in == 0)
    {
        /*  Force LO1 to feed through mode  */
        pInfo->reg[LO1_2] |= 0x40;
        status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO1_2, &pInfo->reg[LO1_2], 1);   /* 0x0D */
    }
    else
    {
	/*
	**  Find RF Band setting
        */
        for (idx=0; idx<10; ++idx)
        {
            if (pInfo->RF_Bands[idx] >= f_in)
            {
                RFBand = 11 - idx;
                break;
            }
        }
    }
	
    /*
    **  Assign in the requested values
    */
    pInfo->AS_Data.f_in = f_in;
    pInfo->AS_Data.f_out = f_out;
	
    /*  Program LO1 and LO2 for a 1st IF frequency of x[i]  */
    pInfo->AS_Data.f_LO1 = CalcLO1Mult(&LO1, &Num1, (UData_t) f_IF1 + f_in, pInfo->AS_Data.f_LO1_Step, pInfo->AS_Data.f_ref);
    pInfo->AS_Data.f_LO2 = Round_fLO(pInfo->AS_Data.f_LO1 - f_out - f_in, pInfo->AS_Data.f_LO2_Step, pInfo->AS_Data.f_ref);
    pInfo->AS_Data.f_LO2 = CalcLO2Mult(&LO2, &Num2, pInfo->AS_Data.f_LO2, pInfo->AS_Data.f_LO2_Step, pInfo->AS_Data.f_ref);
	
    pInfo->reg[LO1C_1] = (U8Data)((RFBand << 4) | (Num1 >> 2));
    pInfo->reg[LO1C_2] = (U8Data)(LO1 & 0xFF);
	
    pInfo->reg[LO2C_1] = (U8Data)((Num2 & 0x000F) | ((Num1 & 0x03) << 4));
    pInfo->reg[LO2C_2] = (U8Data)((Num2 & 0x0FF0) >> 4);
    pInfo->reg[LO2C_3] = (U8Data)(((LO2 << 1) & 0xFE) | ((Num2 & 0x1000) >> 12));
	
    /*  Now write out the computed register values  */
    status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO1C_1, &pInfo->reg[LO1C_1], 5);   /* 0x01 - 0x05 */
	
    if (MT_NO_ERROR(status))
        status |= MT2060_GetLocked((Handle_t) pInfo);
	
    return (status);
}

UData_t MT2060_LocateIF1(Handle_t h, 
						 UData_t f_in, 
						 UData_t f_out, 
						 UData_t* f_IF1, 
						 SData_t* P_max)
{
    UData_t status = MT_OK;
    UData_t i;
    UData_t f_FMF;
    SData_t Pmeas;
    UData_t i_max = 1;

#if defined (__FLOATING_POINT__)
    FData_t x[9], y[9], y2[9];
    FData_t x_max, y_max;
    FData_t xm, xp;
#else
    UData_t x[9];
    UData_t x_lo, x_hi;
    SData_t y[9];
    SData_t y_lo, y_hi, y_meas;
    UData_t dx, xm, xp;
    UData_t n;
#endif
    MT2060_Info_t* pInfo = (MT2060_Info_t*) h;

    printk("--->MT2060_LocateIF1  \n");	   
    printk("--->MT2060_LocateIF1::step1  \n");		

    if (IsValidHandle(pInfo) == 0)
        return MT_INV_HANDLE;
	
    if ((f_IF1 == NULL) || (P_max == NULL))
        return MT_ARG_NULL;
	
    /*  Get a new FMF value from the MT2060  */
    printk("--->MT2060_LocateIF1::Get a new FMF value from the MT2060  \n");
    status |= MT2060_GetFMF(h);
	
    if (MT_IS_ERROR(status))
        return (status);
	
    f_FMF = pInfo->reg[FM_FREQ];
    /*  Enforce limits on FMF so that the LO1 & LO2 limits are not breached  */
    /*  1088 <= f_LO1 <= 2214  */
    /*  1041 <= f_LO2 <= 1310  */
    if (f_FMF < 49)
        f_FMF = 49;
    if (f_FMF < ((f_out / 1000000) + 2))
        f_FMF = ((f_out / 1000000) + 2);
    if (f_FMF > ((f_out / 1000000) + 181))
        f_FMF = ((f_out / 1000000) + 181);
	
    /*  Set the frequencies to be used by the cubic spline  */
    x[0] = 1000000 * (pInfo->AS_Data.f_ref / 16000000) * (f_FMF + 956);
    x[8] = 1000000 * (pInfo->AS_Data.f_ref / 16000000) * (f_FMF + 1212);
    for (i=1; i<8; i++)
        x[i] = 1000000 * (pInfo->AS_Data.f_ref / 16000000) * (f_FMF + 1024 + (15 * i));
	
    /*  Measure the power levels for the cubic spline at the given frequencies */
    for (i=1; ((i<8) && (MT_NO_ERROR(status))); i++)
    {
        /*  Program LO1 and LO2 for a 1st IF frequency of x[i]  */
        status |= MT2060_Program_LOs(pInfo, f_in, (UData_t) x[i], f_out);
		
        if (MT_NO_ERROR(status))
        {
            /*  Ask the demodulator to measure the gain of the MT2060  */
            Pmeas = (SData_t) x[i];
            status |= MT_TunerGain(pInfo->hUserData, &Pmeas);
#if defined (__FLOATING_POINT__)
            y[i] = Pmeas / 100.0;
#else
            y[i] = Pmeas;
#endif
			
            /*  Keep track of the maximum power level measured  */
           if ((i == 1) || (Pmeas > *P_max))
            {
                i_max = i;
                *P_max = Pmeas;
            }
        }
    }
	printk("--->MT2060_LocateIF1::step2  \n");
    /*  Force end-points to be -50 dBc  */
#if defined (__FLOATING_POINT__)
    y[0] = y[8] = (FData_t) *P_max / 100.0 - 50.0;
#else
    y[0] = y[8] = *P_max - 5000;
#endif
	
    if (f_in == 0)                      /*  want to try, even if there were errors  */
    {
        /*  Turn off LO1 feed through mode  */
        pInfo->reg[LO1_2] &= 0xBF;
        status |= MT_WriteSub(pInfo->hUserData, pInfo->address, LO1_2, &pInfo->reg[LO1_2], 1);   /* 0x0D */
    }
	
    if (MT_NO_ERROR(status))
    {
#if defined (__FLOATING_POINT__)
        /*  Calculate the 2nd derivatives for each data point  */
        spline(x, y, y2);
		
        /*  Find the maximum of the cubic-spline curve */
        y_max = findMax(x, y, y2, &x_max);
		
        /*  Move the entire curve such that the peak is at +1.5 dB  */
        for (i=0;i<9;i++)
            y[i]=y[i]-y_max+1.5;
		
        /*  Find the -1.5 dBc points to the left and right of the peak  */
        xm = findRoot(x, y, y2, x_max, x[0]);
        xp = findRoot(x, y, y2, x_max, x[8]);
#else
        /*  The peak of the filter is x[i_max], y[i_max]  */
		
        /*  To Do:  Change y[] measurements type UData_t  */
		
        /*  Move the entire curve such that the peak is at +1.5 dB  */
        for (i=0;i<9;i++)
            y[i] -= (*P_max-150);
		
			/*
			**  Look for the 1.5 dBc (low frequency side)
			**  We are guaranteed that x[0] & x[8] are 50 dB
			**  below the peak measurement.
        */
       i = i_max - 1;
        while (y[i] > 0)
            i--;
		
        /*  The 1.5 dBc point is now bounded by x[i] .. x[i+1]  */
        y_lo = y[i];
        x_lo = x[i];
        y_hi = y[i+1];
        x_hi = x[i+1];
		
        /*
        **
        **               (x_hi, y_hi)
        **  Peak               o--------------o------
        **                    /
        **                  /
        **                /
        **  1.5 dBc ..../............................
        **            /
        **          /
        **         o
        **   (x_lo, y_lo)
        **
        */
        n = 0;                          /*  # times through the loop         */
        dx = (-y_lo < y_hi) ? 10000000 : 5000000;
        while (((x_hi - x_lo) > 5000000) && (++n<3))
        {
            status |= MT2060_Program_LOs(pInfo, f_in, x_hi - dx, f_out);
            if (MT_NO_ERROR(status))
            {
                y_meas = (SData_t) (x_hi - dx);
                status |= MT_TunerGain(pInfo->hUserData, &y_meas);
                if (MT_NO_ERROR(status))
                {
                    y_meas -= (*P_max-150);
                    if (y_meas > 0)
                    {
                        y_hi = y_meas;
                        x_hi = x_hi - dx;
                    }
                    else
                    {
                        y_lo = y_meas;
                        x_lo = x_hi - dx;
                    }
                }
            }
            dx = 5000000;
        }
        /*  Use linear interpolation  */
        xm = x_lo + (-y_lo * (x_hi - x_lo)) / (y_hi - y_lo);
		
        /*
        **  Look for the 1.5 dBc (high frequency side)
        **  We are guaranteed that x[0] & x[8] are 50 dB
        **  below the peak measurement.
        */
        i = i_max + 1;
        while (y[i] > 0)
            i++;
		
        /*  The 1.5 dBc point is now bounded by x[i-1] .. x[i]  */
        y_lo = y[i-1];
        x_lo = x[i-1];
        y_hi = y[i];
        x_hi = x[i];
		
        n = 0;                          /*  # times through the loop         */
        dx = (-y_hi > y_lo) ? 5000000 : 10000000;
        while (((x_hi - x_lo) > 5000000) && (++n<3))
        {
            status |= MT2060_Program_LOs(pInfo, f_in, x_lo + dx, f_out);
            if (MT_NO_ERROR(status))
            {
                y_meas = (SData_t) (x_lo + dx);
                status |= MT_TunerGain(pInfo->hUserData, &y_meas);
                if (MT_NO_ERROR(status))
                {
                    y_meas -= (*P_max-150);
                    if (y_meas > 0)
                    {
                        y_lo = y_meas;
                        x_lo += dx;
                    }
                    else
                    {
                        y_hi = y_meas;
                        x_hi = x_lo + dx;
                    }
                }
            }
            dx = 5000000;
        }
        /*  Use linear interpolation  */
        xp = x_lo + (y_lo * (x_hi - x_lo)) / (y_lo - y_hi);
#endif
        *f_IF1 = (UData_t) ((xm + xp) / 2);
    }
    printk("--->MT2060_LocateIF1::end  \n");	
    return (status);

}
