#ifndef __TUNER_BASE_H
#define __TUNER_BASE_H


/**

@file

@brief   Tuner base module definition

Tuner base module definitions contains tuner module structure, demod funciton pointers, and tuner definitions.

*/

#include "dvb-usb.h"
#include "i2c_bridge.h"





/**

@defgroup   TUNER_BASE_MODULE   Tuner base module

One can use tuner base module interface implemented by function pointers to manipulate tuner.
All tuner modules should be derived from tuner base module.

*/





/**

@addtogroup   TUNER_MODULE_STRUCTURE

*/
/// @{

/// tuner module pre-definition
typedef struct TUNER_MODULE_TAG TUNER_MODULE;

/// @}





/**

@defgroup   TUNER_MANIPULATING_FUNCTIONS   Tuner manipulating functions
@ingroup    TUNER_BASE_MODULE

One can manipulate tuner through tuner manipulating functions.

*/
/// @{





/**

@brief   Tuner I2C device address setting function pointer

One can use TUNER_FP_SET_DEVICE_ADDR() to set tuner I2C device address.


@param [in]   pTuner       The tuner module pointer
@param [in]   DeviceAddr   Tuner I2C device address for setting


@note \n
	-# Tuner building function will set TUNER_FP_SET_DEVICE_ADDR() with the corresponding function.

*/
typedef void
(*TUNER_FP_SET_DEVICE_ADDR)(
	TUNER_MODULE *pTuner,
	unsigned char DeviceAddr
	);





/**

@brief   Tuner type getting function pointer

One can use TUNER_FP_GET_TUNER_TYPE() to get tuner type.


@param [in]    pTuner       The tuner module pointer
@param [out]   pTunerType   Pointer to an allocated memory for storing tuner type


@note \n
	-# Tuner building function will set TUNER_FP_GET_TUNER_TYPE() with the corresponding function.


@see   TUNER_TYPE

*/
typedef void
(*TUNER_FP_GET_TUNER_TYPE)(
	TUNER_MODULE *pTuner,
	int *pTunerType
	);





/**

@brief   Tuner I2C device address getting function pointer

One can use TUNER_FP_GET_DEVICE_ADDR() to get tuner I2C device address.


@param [in]    pTuner        The tuner module pointer
@param [out]   pDeviceAddr   Pointer to an allocated memory for storing tuner I2C device address


@retval   FUNCTION_SUCCESS   Get tuner device address successfully.
@retval   FUNCTION_ERROR     Get tuner device address unsuccessfully.


@note \n
	-# Tuner building function will set TUNER_FP_GET_DEVICE_ADDR() with the corresponding function.

*/
typedef int
(*TUNER_FP_GET_DEVICE_ADDR)(
	TUNER_MODULE *pTuner,
	unsigned char *pDeviceAddr
	);





/**

@brief   Tuner initializing function pointer

One can use TUNER_FP_INITIALIZE() to initialie tuner.


@param [in]   pTuner   The tuner module pointer


@retval   FUNCTION_SUCCESS   Initialize tuner successfully.
@retval   FUNCTION_ERROR     Initialize tuner unsuccessfully.


@note \n
	-# Tuner building function will set TUNER_FP_INITIALIZE() with the corresponding function.

*/
typedef int
(*TUNER_FP_INITIALIZE)(
	struct dvb_usb_device*        dib,
	TUNER_MODULE *pTuner
	);





/**

@brief   Tuner RF frequency setting function pointer

One can use TUNER_FP_SET_RF_FREQ_HZ() to set tuner RF frequency in Hz.


@param [in]   pTuner     The tuner module pointer
@param [in]   RfFreqHz   RF frequency in Hz for setting


@retval   FUNCTION_SUCCESS   Set tuner RF frequency successfully.
@retval   FUNCTION_ERROR     Set tuner RF frequency unsuccessfully.


@note \n
	-# Tuner building function will set TUNER_FP_SET_RF_FREQ_HZ() with the corresponding function.

*/
typedef int
(*TUNER_FP_SET_RF_FREQ_HZ)(
	struct dvb_usb_device*        dib,
	TUNER_MODULE *pTuner,
	unsigned long RfFreqHz
	);





/**

@brief   Tuner RF frequency getting function pointer

One can use TUNER_FP_GET_RF_FREQ_HZ() to get tuner RF frequency in Hz.


@param [in]   pTuner      The tuner module pointer
@param [in]   pRfFreqHz   Pointer to an allocated memory for storing demod RF frequency in Hz


@retval   FUNCTION_SUCCESS   Get tuner RF frequency successfully.
@retval   FUNCTION_ERROR     Get tuner RF frequency unsuccessfully.


@note \n
	-# Tuner building function will set TUNER_FP_GET_RF_FREQ_HZ() with the corresponding function.

*/
typedef int
(*TUNER_FP_GET_RF_FREQ_HZ)(
	struct dvb_usb_device*        dib,
	TUNER_MODULE *pTuner,
	unsigned long *pRfFreqHz
	);





/// @}





/**

@defgroup   TUNER_MODULE_STRUCTURE   Tuner module structure
@ingroup    TUNER_BASE_MODULE

Tuner module structure contains manipulating functions and private variables .

*/
/// @{



/// Tuner module structure
struct TUNER_MODULE_TAG
{
	// Private variables
	int           TunerType;									///<   Tuner type
	unsigned char DeviceAddr;									///<   Tuner I2C device address
	unsigned long RfFreqHz;										///<   Tuner RF frequency in Hz

	int IsDeviceAddrSet;										///<   Tuner I2C device address (setting status)
	int IsRfFreqHzSet;											///<   Tuner RF frequency in Hz (setting status)

	void *pExtra;												///<   Tuner extra module used by driving module
	BASE_INTERFACE_MODULE *pBaseInterface;						///<   Base interface module
	I2C_BRIDGE_MODULE *pI2cBridge;								///<   I2C bridge module


	// Tuner manipulating functions
	TUNER_FP_SET_DEVICE_ADDR         SetDeviceAddr;				///<   Tuner I2C device address setting function pointer

	TUNER_FP_GET_TUNER_TYPE          GetTunerType;				///<   Tuner type getting function pointer
	TUNER_FP_GET_DEVICE_ADDR         GetDeviceAddr;				///<   Tuner I2C device address getting function pointer

	TUNER_FP_INITIALIZE              Initialize;				///<   Tuner initializing function pointer
	TUNER_FP_SET_RF_FREQ_HZ          SetRfFreqHz;				///<   Tuner RF frequency setting function pointer
	TUNER_FP_GET_RF_FREQ_HZ          GetRfFreqHz;				///<   Tuner RF frequency getting function pointer

};



/// @}

















#endif
